@@ -1,12 +1,12 @@
 /*
   This file is Copyright © 1994-1995 Olivier Montanuy,
                Copyright © 1999-2005 André Majorel,
-               Copyright © 2006-2018 contributors to the DeuTex project.
+               Copyright © 2006-2019 contributors to the DeuTex project.
 
   DeuTex incorporates code derived from DEU 5.21 that was put in the
   public domain in 1994 by Raphaël Quinet and Brendon Wyber.
 
-  SPDX-License-Identifier: GPL-2.0+
+  SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "deutex.h"
@@ -43,7 +43,7 @@ static bool CMPOcopyFromWAD(int32_t * size, struct WADINFO *rwad,
 {
     static struct WADINFO pwad;
     int16_t entry;
-    if (MakeFileName(file, DataDir, Dir, "", filenam, "WAD") != true)
+    if (!MakeFileName(file, DataDir, Dir, "", filenam, "WAD"))
         return false;
     WADRopenR(&pwad, file);
     entry = WADRfindEntry(&pwad, nam);
@@ -68,17 +68,17 @@ static int16_t CMPOloadPic(int32_t * size, struct WADINFO *rwad,
                            int16_t OfsY)
 {
     int res = PICNONE;
-    if (MakeFileName(file, DataDir, Dir, "", filenam, "ppm") == true)
+    if (MakeFileName(file, DataDir, Dir, "", filenam, "ppm"))
         res = PICPPM;
-    else if (MakeFileName(file, DataDir, Dir, "", filenam, "bmp") == true)
+    else if (MakeFileName(file, DataDir, Dir, "", filenam, "bmp"))
         res = PICBMP;
 #ifdef HAVE_LIBPNG
-    else if (MakeFileName(file, DataDir, Dir, "", filenam, "png") == true)
+    else if (MakeFileName(file, DataDir, Dir, "", filenam, "png"))
         res = PICPNG;
 #endif
-    else if (MakeFileName(file, DataDir, Dir, "", filenam, "gif") == true)
+    else if (MakeFileName(file, DataDir, Dir, "", filenam, "gif"))
         res = PICGIF;
-    else if (CMPOcopyFromWAD(size, rwad, DataDir, Dir, nam, filenam) == true)
+    else if (CMPOcopyFromWAD(size, rwad, DataDir, Dir, nam, filenam))
         return PICWAD;
     if (res != PICNONE)
         *size = PICsaveInWAD(rwad, file, Type, OfsX, OfsY, res);
@@ -93,6 +93,28 @@ static int16_t CMPOloadPic(int32_t * size, struct WADINFO *rwad,
     return res;
 }
 
+/*
+** find a raw picture, for TX_START support.
+*/
+static bool CMPOloadRawPic(int32_t * size, struct WADINFO *rwad,
+                           char *file, const char *DataDir,
+                           const char *Dir, const char *nam,
+                           const char *filenam)
+{
+    /* most ports support PNG, ZDoom also supports JPEG */
+    if (MakeFileName(file, DataDir, Dir, "", filenam, "jpeg") ||
+        MakeFileName(file, DataDir, Dir, "", filenam, "jpg") ||
+        MakeFileName(file, DataDir, Dir, "", filenam, "png")) {
+
+        *size = WADRwriteLump(rwad, file);
+        return true;
+    }
+
+    /* file has the last filename tried, i.e. NAME.png */
+    Warning("PC91", "could not find file %s, .gif, or .jpeg", file);
+    return false;
+}
+
 struct WADINFO *CMPOrwad;
 const char *CMPOwadout = NULL;
 void CMPOerrorAction(void)
@@ -156,13 +178,13 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
     if (select & BLEVEL) {
         if (TXTseekSection(TXT, "LEVELS")) {
             while (TXTentryParse
-                   (name, filenam, &X, &Y, &Repeat, TXT, false) == true) {
+                   (name, filenam, &X, &Y, &Repeat, TXT, false)) {
                 p = IDENTlevel(name);
                 if (p < 0)
                     ProgError("CM11", "Illegal level name %s",
                               lump_name(name));
-                if (MakeFileName
-                    (file, DataDir, "LEVELS", "", filenam, "WAD") != true)
+                if (!MakeFileName
+                    (file, DataDir, "LEVELS", "", filenam, "WAD"))
                     ProgError("CM12", "Can't find level WAD %s",
                               fname(file));
                 Detail("CM13", "Reading level WAD file %s", fname(file));
@@ -183,7 +205,7 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
 
         if (TXTseekSection(TXT, "LUMPS")) {
             while (TXTentryParse
-                   (name, filenam, &X, &Y, &Repeat, TXT, false) == true) {
+                   (name, filenam, &X, &Y, &Repeat, TXT, false)) {
                 if (strcmp(name, "PLAYPAL") == 0) {
                     FILE *playpal_fp;
 
@@ -229,13 +251,12 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
         if (TXTseekSection(TXT, "LUMPS")) {
             Phase("CM30", "Making lumps");
             while (TXTentryParse
-                   (name, filenam, &X, &Y, &Repeat, TXT, false) == true) {
-                if (Repeat != true) {
+                   (name, filenam, &X, &Y, &Repeat, TXT, false)) {
+                if (!Repeat) {
                     WADRalign4(&rwad);  /*align entry on int32_t word */
                     start = WADRposition(&rwad);
                     if (MakeFileName
-                        (file, DataDir, "LUMPS", "", filenam,
-                         "LMP") == true) {
+                        (file, DataDir, "LUMPS", "", filenam, "LMP")) {
                         size = WADRwriteLump(&rwad, file);
                     } else {
                         Picture =
@@ -243,10 +264,9 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
                                         "LUMPS", name, filenam, PLUMP, X,
                                         Y);
                         if (Picture == PICNONE)
-                            if (CMPOcopyFromWAD
+                            if (!CMPOcopyFromWAD
                                 (&size, &rwad, DataDir, "LUMPS", name,
-                                 filenam)
-                                != true)
+                                 filenam))
                                 ProgError("CM31",
                                           "Can't find lump or picture file %s",
                                           file);
@@ -285,17 +305,16 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
             FoundOne = false;
             /*read TEXTURES composing TEXTURE1 */
             while (TXTentryParse
-                   (name, filenam, &X, &Y, &Repeat, TXT, false) == true) {
+                   (name, filenam, &X, &Y, &Repeat, TXT, false)) {
                 if (MakeFileName
-                    (file, DataDir, "TEXTURES", "", filenam, "TXT") == true) {
+                    (file, DataDir, "TEXTURES", "", filenam, "TXT")) {
                     Detail("CM52", "Reading texture file %s", fname(file));
                     TXUreadTexFile(file, true);
                     NeedPNAME = true;
                     FoundOne = true;
                 } else
                     if (MakeFileName
-                        (file, DataDir, "TEXTURES", "", name,
-                         "WAD") == true) {
+                        (file, DataDir, "TEXTURES", "", name, "WAD")) {
                         Detail("CM53", "Reading texture WAD %s", fname(file));
                         WADRopenR(&pwad, file);
                         entry = WADRfindEntry(&pwad, "TEXTURE1");
@@ -312,7 +331,7 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
                         ProgError("CM54", "Can't find texture list %s", file);
             }
             /*write texture */
-            if (FoundOne == true) {
+            if (FoundOne) {
                 WADRalign4(&rwad);      /*align entry on int32_t word */
                 start = WADRposition(&rwad);
                 size = TXUwriteTEXTUREtoWAD(&rwad);
@@ -338,17 +357,16 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
             FoundOne = false;
             /*read TEXTURES composing TEXTURE2 */
             while (TXTentryParse
-                   (name, filenam, &X, &Y, &Repeat, TXT, false) == true) {
+                   (name, filenam, &X, &Y, &Repeat, TXT, false)) {
                 if (MakeFileName
-                    (file, DataDir, "TEXTURES", "", filenam, "TXT") == true) {
+                    (file, DataDir, "TEXTURES", "", filenam, "TXT")) {
                     Detail("CM57", "Reading texture file %s", fname(file));
                     TXUreadTexFile(file, true);
                     NeedPNAME = true;
                     FoundOne = true;
                 } else
                     if (MakeFileName
-                        (file, DataDir, "TEXTURES", "", name,
-                         "WAD") == true) {
+                        (file, DataDir, "TEXTURES", "", name, "WAD")) {
                         Detail("CM58", "Reading texture WAD %s", fname(file));
                         WADRopenR(&pwad, file);
                         entry = WADRfindEntry(&pwad, "TEXTURE2");
@@ -365,7 +383,7 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
                         ProgError("CM59", "Can't find texture list %s", file);
             }
             /*write texture */
-            if (FoundOne == true) {
+            if (FoundOne) {
                 WADRalign4(&rwad);      /*align entry on int32_t word */
                 start = WADRposition(&rwad);
                 size = TXUwriteTEXTUREtoWAD(&rwad);
@@ -388,6 +406,47 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
         }
     }
     /*
+    ** TX_START
+    ** textures  (patch or PNG format)
+    ** TX_END
+    */
+    if (select & BTEXTUR) {
+        start = size = 0;
+        if (TXTseekSection(TXT, "TX_START")) {
+            Phase("CM59", "Making TX_START textures");
+            FoundOne = false;
+            while (TXTentryParse
+                   (name, filenam, &X, &Y, &Repeat, TXT, true)) {
+                /* only add markers if we have some textures */
+                if (!FoundOne) {
+                    WADRalign4(&rwad);  /*align entry on int32_t word */
+                    start = WADRposition(&rwad);
+                    WADRdirAddEntry(&rwad, start, 0L, "TX_START");
+                    FoundOne = true;
+                }
+
+                WADRalign4(&rwad);  /*align entry on int32_t word */
+                start = WADRposition(&rwad);
+                size = 0;
+
+                /* when X value is > 0, insert as a raw lump (no conversion) */
+                if (X > 0) {
+                    CMPOloadRawPic(&size, &rwad, file, DataDir, "TX_START",
+                                   name, filenam);
+                } else {
+                    CMPOloadPic(&size, &rwad, file, DataDir, "TX_START",
+                                name, filenam, PGRAPH, X, Y);
+                }
+                WADRdirAddEntry(&rwad, start, size, name);
+            }
+            if (FoundOne) {
+                WADRalign4(&rwad);
+                start = WADRposition(&rwad);
+                WADRdirAddEntry(&rwad, start, 0L, "TX_END");
+            }
+        }
+    }
+    /*
     **
     **   sounds. all sounds entries
     */
@@ -396,25 +455,23 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
         if (TXTseekSection(TXT, "SOUNDS")) {
             Phase("CM60", "Making sounds");
             while (TXTentryParse
-                   (name, filenam, &X, &Y, &Repeat, TXT, false) == true) {
-                if (Repeat != true) {
+                   (name, filenam, &X, &Y, &Repeat, TXT, false)) {
+                if (!Repeat) {
                     WADRalign4(&rwad);  /*align entry on int32_t word */
                     start = WADRposition(&rwad);
                     if (MakeFileName
-                        (file, DataDir, "SOUNDS", "", filenam,
-                         "TXT") == true) {
+                        (file, DataDir, "SOUNDS", "", filenam, "TXT")) {
                         size = SNDcopyPCSoundInWAD(&rwad, file);
                         Detail("CM62", "Reading PC sound from file %s",
                                fname(file));
                     } else {
                         if (MakeFileName
-                            (file, DataDir, "SOUNDS", "", filenam,
-                             "WAV") == true) {
+                            (file, DataDir, "SOUNDS", "", filenam, "WAV")) {
                             size = SNDcopyInWAD(&rwad, file, SNDWAV);
                         } else {
-                            if (CMPOcopyFromWAD
+                            if (!CMPOcopyFromWAD
                                 (&size, &rwad, DataDir, "SOUNDS", name,
-                                 filenam) != true) {
+                                 filenam)) {
                                 ProgError("CM63",
                                           "Can't find sound %s, WAV",
                                           file);
@@ -437,28 +494,26 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
         if (TXTseekSection(TXT, "MUSICS")) {
             Phase("CM65", "Making musics");
             while (TXTentryParse
-                   (name, filenam, &X, &Y, &Repeat, TXT, false) == true) {
-                if (Repeat != true) {
+                   (name, filenam, &X, &Y, &Repeat, TXT, false)) {
+                if (!Repeat) {
                     WADRalign4(&rwad);  /*align entry on int32_t word */
                     start = WADRposition(&rwad);
                     /*Music */
                     if (MakeFileName
-                        (file, DataDir, "MUSICS", "", filenam,
-                         "MUS") == true) {
+                        (file, DataDir, "MUSICS", "", filenam, "MUS")) {
                         size = WADRwriteLump(&rwad, file);
                         Detail("CM66", "Reading music file %s",
                                fname(file));
                     } else
                         if (MakeFileName
-                            (file, DataDir, "MUSICS", "", filenam,
-                             "MID") == true) {
+                            (file, DataDir, "MUSICS", "", filenam, "MID")) {
                             size = WADRwriteLump(&rwad, file);
                             Detail("CM68", "Reading music file %s",
                                    fname(file));
                         } else
-                            if (CMPOcopyFromWAD
+                            if (!CMPOcopyFromWAD
                                 (&size, &rwad, DataDir, "MUSICS", name,
-                                 filenam) != true)
+                                 filenam))
                                 ProgError("CM67", "Can't find music %s", file);
                 }
                 WADRdirAddEntry(&rwad, start, size, name);
@@ -472,9 +527,9 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
         start = size = 0;
         if (TXTseekSection(TXT, "GRAPHICS")) {
             Phase("CM70", "Making graphics");
-            while (TXTentryParse(name, filenam, &X, &Y, &Repeat, TXT, true)
-                   == true) {
-                if (Repeat != true) {
+            while (TXTentryParse
+                   (name, filenam, &X, &Y, &Repeat, TXT, true)) {
+                if (!Repeat) {
                     WADRalign4(&rwad);  /*align entry on int32_t word */
                     start = WADRposition(&rwad);
                     Picture =
@@ -497,12 +552,12 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
             Phase("CM75", "Making sprites");
             FoundOne = false;
             while (TXTentryParse
-                   (name, filenam, &X, &Y, &Repeat, TXT, true) == true) {
+                   (name, filenam, &X, &Y, &Repeat, TXT, true)) {
                 /* first sprite seen? */
-                if ((Repeat != true) || (FoundOne != true)) {
+                if (!Repeat || !FoundOne) {
                     WADRalign4(&rwad);  /*align entry on int32_t word */
                     start = WADRposition(&rwad);
-                    if (FoundOne != true) {
+                    if (!FoundOne) {
                         if (type == IWAD)
                             WADRdirAddEntry(&rwad, start, 0L, "S_START");
                         else
@@ -514,10 +569,10 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
                 }
                 WADRdirAddEntry(&rwad, start, size, name);
             }
-            if (FoundOne == true) {
+            if (FoundOne) {
                 WADRalign4(&rwad);
                 start = WADRposition(&rwad);
-                if ((type == IWAD) || (George == true))
+                if ((type == IWAD) || George)
                     WADRdirAddEntry(&rwad, start, 0L, "S_END");
                 else
                     WADRdirAddEntry(&rwad, start, 0L, "SS_END");
@@ -540,12 +595,12 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
         start = size = 0;
         if (TXTseekSection(TXT, "PATCHES")) {
             Phase("CM80", "Making patches");
-            while (TXTentryParse(name, filenam, &X, &Y, &Repeat, TXT, true)
-                   == true) {
-                if ((Repeat != true) || (FoundOne != true)) {
+            while (TXTentryParse
+                   (name, filenam, &X, &Y, &Repeat, TXT, true)) {
+                if (!Repeat || !FoundOne) {
                     WADRalign4(&rwad);  /*align entry on int32_t word */
                     start = WADRposition(&rwad);
-                    if (FoundOne == false) {
+                    if (!FoundOne) {
                         if (type == IWAD) {
                             WADRdirAddEntry(&rwad, start, 0L, "P_START");
                             WADRdirAddEntry(&rwad, start, 0L, "P1_START");
@@ -565,7 +620,7 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
         */
         nbPatchs = PNMgetNbOfPatch();
         for (p = 0; p < nbPatchs; p++) {
-            if (PNMisNew(p) != true) {
+            if (!PNMisNew(p)) {
                 continue;       /*if old patch, forget it */
             }
             PNMgetPatchName(name, p);
@@ -587,7 +642,7 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
                                 name, filenam, PPATCH, INVALIDINT,
                                 INVALIDINT);
                 if (Picture != PICNONE) {
-                    if (FoundOne == false) {
+                    if (!FoundOne) {
                         Phase("CM82", "Making patches");
                         if (type == IWAD) {
                             WADRdirAddEntry(&rwad, start, 0L, "P_START");
@@ -600,7 +655,7 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
                 }
             }
         }
-        if (FoundOne == true) {
+        if (FoundOne) {
             WADRalign4(&rwad);  /*align entry on int32_t word */
             start = WADRposition(&rwad);
             if (type == IWAD) {
@@ -629,13 +684,13 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
             Phase("CM85", "Making flats");
             FoundOne = false;
             while (TXTentryParse
-                   (name, filenam, &X, &Y, &Repeat, TXT, false) == true) {
-                if ((Repeat != true) || (FoundOne != true)) {
+                   (name, filenam, &X, &Y, &Repeat, TXT, false)) {
+                if (!Repeat || !FoundOne) {
                     /*align entry on int32_t word */
                     WADRalign4(&rwad);
 
                     start = WADRposition(&rwad);
-                    if (FoundOne == false) {
+                    if (!FoundOne) {
                         if (type == IWAD) {
                             WADRdirAddEntry(&rwad, start, 0L, "F_START");
                             WADRdirAddEntry(&rwad, start, 0L, "F1_START");
@@ -648,7 +703,7 @@ void CMPOmakePWAD(const char *doomwad, WADTYPE type, const char *PWADname,
                 }
                 WADRdirAddEntry(&rwad, start, size, name);
             }
-            if (FoundOne == true) {
+            if (FoundOne) {
                 start = WADRposition(&rwad);
                 if (type == IWAD) {
                     WADRdirAddEntry(&rwad, start, 0L, "F1_END");
