@@ -1,12 +1,12 @@
 /*
   This file is Copyright © 1994-1995 Olivier Montanuy,
                Copyright © 1999-2005 André Majorel,
-               Copyright © 2006-2018 contributors to the DeuTex project.
+               Copyright © 2006-2019 contributors to the DeuTex project.
 
   DeuTex incorporates code derived from DEU 5.21 that was put in the
   public domain in 1994 by Raphaël Quinet and Brendon Wyber.
 
-  SPDX-License-Identifier: GPL-2.0+
+  SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "deutex.h"
@@ -339,10 +339,10 @@ void CheckTexture(char *tex, bool IsDef)
     case '\0':
         break;
     default:
-        if (IsDef == true) {    /*if we only wish to declare the tex */
+        if (IsDef) {    /*if we only wish to declare the tex */
             TXUfakeTex(Name);
         } else {
-            if (TXUexist(Name) == false)
+            if (!TXUexist(Name))
                 Output("Warning: undefined sidedef %s\n", lump_name(Name));
         }
     }
@@ -660,13 +660,13 @@ void XTRcompakWAD(const char *DataDir, const char *wadin,
     bbas = (char *) Malloc(MEMORYCACHE);
     btst = (char *) Malloc(MEMORYCACHE);
     for (bas = 0; bas < pnb; bas++)
-        if (psame[bas] == false) {      /*skip already treated */
+        if (!psame[bas]) {      /*skip already treated */
             size = pdir[bas].size;
             if (pdir[bas].start <= 8)
                 continue;
             if (size < 1)
                 continue;
-            if ((size >= 8) && (ShowIdx == true)) {
+            if ((size >= 8) && ShowIdx) {
                 WADRseek(&pwad, pdir[bas].start);
                 WADRreadBytes(&pwad, bbas, 8);
                 ofsx = ((bbas[5] << 8) & 0xFF00) + (bbas[4] & 0xFF);
